//----------------------------------------------------------------------------------
// File:   InstancingTests.cpp
// Author: Bryan Dudash
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2008 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#pragma once

#include <vector>

class InstancingTestHarness
{
public:
    InstancingTestHarness();
    void InitializeExperiments(InstancingTests*pIT, 
                                  DWORD maxInstances, DWORD maxLoad, 
                                  DWORD instSteps, DWORD loadSteps, 
                                  bool bLogIncrememnt, int frameSmooth,
                                  LPCWSTR szTag);
    void FrameUpdate(InstancingTests*pSO,float elapsedTime);
    void FrameRender(InstancingTests*pSO,float elapsedTime);
    void WriteOutData(InstancingTests*pSO);
    bool GetSimulationComplete(){return m_complete;};

    struct SimulationTestEvent
    {
        DWORD instances;
        DWORD cpuload;
        unsigned char useinstancing;
    };

    struct SimulationTestResult
    {
        unsigned char useinstancing;
        DWORD instances;
        DWORD cpuload;
        float ms;
    };

    SimulationTestEvent &GetCurrentEvent(){return m_events[m_currentEvent];}

    std::vector<SimulationTestResult> m_results;
    std::vector<SimulationTestEvent> m_events;

    bool m_complete;

    int m_currentEvent;
    int m_accumFrame;
    float m_accumTime;

    int m_FrameSmoothingFactor;
    char m_szFilename[MAX_PATH];
   
};