//----------------------------------------------------------------------------------
// File:   InstancingTests.cpp
// Author: Bryan Dudash
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2008 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#pragma once

#include "sdkmesh_old.h"
#define INTERNAL_MAX_INSTANCES 1365
#define INTERNAL_MAX_BUFFERS 100

const D3D10_INPUT_ELEMENT_DESC VertexLayout[] =
{
	{ "POSITION",	0, DXGI_FORMAT_R32G32B32_FLOAT,		0, 0,  D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ "NORMAL",		0, DXGI_FORMAT_R32G32B32_FLOAT,		0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ "TEXCOORD",	0, DXGI_FORMAT_R32G32_FLOAT,		0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ "TANGENT",	0, DXGI_FORMAT_R32G32B32_FLOAT,		0, 32, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ "BITANGENT",	0, DXGI_FORMAT_R32G32B32_FLOAT,		0, 44, D3D10_INPUT_PER_VERTEX_DATA, 0 },
};

const D3D10_INPUT_ELEMENT_DESC InstancedVertexLayout[] =
{
	{ "POSITION",	0, DXGI_FORMAT_R32G32B32_FLOAT,		0, 0,  D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ "NORMAL",		0, DXGI_FORMAT_R32G32B32_FLOAT,		0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ "TEXCOORD",	0, DXGI_FORMAT_R32G32_FLOAT,		0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ "TANGENT",	0, DXGI_FORMAT_R32G32B32_FLOAT,		0, 32, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ "BITANGENT",	0, DXGI_FORMAT_R32G32B32_FLOAT,		0, 44, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    { "WORLD",	    0, DXGI_FORMAT_R32G32B32A32_FLOAT,  1, 0,  D3D10_INPUT_PER_INSTANCE_DATA, 1 },
    { "WORLD",	    1, DXGI_FORMAT_R32G32B32A32_FLOAT,  1, 16,  D3D10_INPUT_PER_INSTANCE_DATA, 1 },
    { "WORLD",      2, DXGI_FORMAT_R32G32B32A32_FLOAT,  1, 32,  D3D10_INPUT_PER_INSTANCE_DATA, 1 },
};

/*
	Wraps up a little test suite that will test performance of instancing versus single draw calls.
*/
class InstancingTests
{
public:

	struct Vertex
	{
		D3DXVECTOR3 pos;
		D3DXVECTOR3 norm;
		D3DXVECTOR2 tex;
		D3DXVECTOR3 tangent;
		D3DXVECTOR3 binorm;
	};

	struct InstanceData
	{
		D3DXVECTOR4 world1;			// the world transform for this matrix row 1
		D3DXVECTOR4 world2;			// the world transform for this matrix row 2
		D3DXVECTOR4 world3;			// the world transform for this matrix row 3 (row 4 is implicit)
	};

	InstancingTests();
	~InstancingTests();

	HRESULT Initialize(ID3D10Device* pd3dDevice);
	HRESULT Update(float time);
	unsigned int Render(ID3D10Device* pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time);
	void Release();

	void SetUseInstancing(bool bUseInstancing){m_bUseInstancing = bUseInstancing;m_bInstanceBufferNeedsUpdate=true;}
	bool UseInstancing(){return m_bUseInstancing;}

    void SetTraditionalInstancing(bool bTraditionalInstancing){m_bTraditionalInstancing = bTraditionalInstancing;m_bInstanceBufferNeedsUpdate=true;}
    bool TraditionalInstancing(){return m_bTraditionalInstancing;}

	void SetNumInstanced(int num){m_numInstances = min(num,NumMaxInstances());m_bInstanceBufferNeedsUpdate = true;}
	int NumInstances(){return m_numInstances;}
    int NumMaxInstances(){return INTERNAL_MAX_BUFFERS*INTERNAL_MAX_INSTANCES - 1;}

	void SetNumPolys(int num){m_numPolys = num;}
	int NumPolys(){return m_numPolys;}

    int NumMeshPolys() {return m_numIndices / 3;}

    void SetMotionActive(bool motion){m_bMoveInstances = motion;}
    bool MotionActive() {return m_bMoveInstances;}

    void SetScale(float scale);

    void SetCPULoad(int ms){m_cpuLoad = ms;}
    int GetCPULoad(){return m_cpuLoad;}

    void LoadNewMesh(LPCWSTR filename);

protected:

	unsigned int Draw(ID3D10Device *pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time);
	unsigned int DrawStreamInstanced(ID3D10Device *pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time);
    unsigned int DrawConstantsInstanced(ID3D10Device *pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj, float time);
    

	HRESULT CreateInstanceBuffers();
	HRESULT UpdateInstanceData( float time );
	
	D3DXMATRIX WorldMatrixFor(int instance);
	void SetWorldMatrixFor(int instance, D3DXMATRIX &mWorld);

    void LoadTexture(const wchar_t *filename,ID3D10Texture2D**pTex,ID3D10ShaderResourceView**pSRV);

	int m_numPolys;
	int m_numInstances;
	bool m_bUseInstancing;
    bool m_bTraditionalInstancing;
    bool m_bMoveInstances;
    bool m_bInstanceBufferNeedsUpdate;
    float m_fScale;
    int m_cpuLoad;

	D3DXVECTOR3 m_objectExtents;

	InstanceData m_instanceData[INTERNAL_MAX_BUFFERS*INTERNAL_MAX_INSTANCES];
    D3DXMATRIX m_initialMatrix[INTERNAL_MAX_BUFFERS*INTERNAL_MAX_INSTANCES];

	int m_numVertices;
	int m_numIndices;

	// D3D10 Rendering vars (Release needed)
    CDXUTMesh10             Mesh;
	ID3D10Device*			m_pd3dDevice;
	ID3D10Effect*			m_pEffect10;
	ID3D10InputLayout*		m_pInputLayout;
    ID3D10InputLayout*		m_pInstancedInputLayout;
	ID3D10Buffer*			m_pMeshBuffer;
	ID3D10Buffer*			m_pMeshIndexBuffer;
	ID3D10Buffer*			m_pInstanceDataBuffer[INTERNAL_MAX_BUFFERS];
    ID3D10Buffer*			m_pTotalInstanceDataBuffer;
    ID3D10Texture2D*        m_pDiffuseTexture;
    ID3D10ShaderResourceView*m_pDiffuseSRV;
    ID3D10Texture2D*        m_pNormalTexture;
    ID3D10ShaderResourceView*m_pNormalSRV;

	// No need to Release these vars
	ID3D10EffectTechnique*  m_pTechInstanced;
    ID3D10EffectTechnique*  m_pTechInstancedConstants;
	ID3D10EffectTechnique*  m_pTechNormal;

	ID3D10EffectMatrixVariable*			m_pWorldVar;
	ID3D10EffectMatrixVariable*			m_pWorldViewProjectionVar;
    ID3D10EffectShaderResourceVariable* m_pDiffuse;
    ID3D10EffectShaderResourceVariable* m_pNormalMap;
	ID3D10EffectConstantBuffer*			m_pInstanceDataConstBuffer;
    

private:

};