//----------------------------------------------------------------------------------
// File:   InstancingTests.cpp
// Author: Bryan Dudash
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2008 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#include "DXUT.h"
#include "DXUTcamera.h"
#include "DXUTgui.h"
#include "DXUTSettingsDlg.h"
#include <string>
#include "resource.h"
#include "InstancingTests.h"
#include "InstancingTestHarness.h"
#include "TextureLibray.h"
#include <vector>
#include "stdio.h"
#include <iostream>
#include <fstream>

#include "SDKmisc.h"
#pragma warning (disable:4996)

float g_Time = 0.0f;

CFirstPersonCamera      g_Camera;               // A model viewing camera
ID3DX10Font*            g_pFont10 = NULL;       
ID3DX10Sprite*          g_pSprite10 = NULL;
CDXUTTextHelper*        g_pTxtHelper = NULL;
unsigned int            g_numDrawCalls = 0;
int                     g_activeMesh = -1;


CDXUTDialogResourceManager      g_DialogResourceManager; // manager for shared resources of dialogs
CDXUTDialog                     g_SampleUI;             // dialog for sample specific controls
CDXUTDialog                     g_HUD;
CDXUTDialog                     g_BenchmarkUI;             // dialog for sample specific controls
CD3DSettingsDlg                 g_SettingsDlg;

InstancingTests                 g_instancing;
InstancingTestHarness           g_testHarness;

#define IDC_TOGGLEFULLSCREEN        1
#define IDC_TOGGLEREF               2
#define IDC_CHANGEDEVICE            3

#define IDC_NUM_INSTANCES_STATIC	5
#define IDC_NUM_INSTANCES			6
#define IDC_USE_INSTANCING			7
#define IDC_USE_ANIMATION			8
#define IDC_CPU_LOAD_STATIC         9
#define IDC_CPU_LOAD                10
#define IDC_MOTION_ACTIVE           11
#define IDC_MESHFILE_STATIC         12
#define IDC_MESHFILE                13
#define IDC_OPENMESHFILE            14
#define IDC_SCALE_STATIC            15
#define IDC_SCALE                   16
#define IDC_TRADITIONAL_INSTANCING  17
#define IDC_BENCHMARKSTART          18

#define IDC_BUI_START               50
#define IDC_BUI_MAXINST_STATIC      51
#define IDC_BUI_MAXINST             52
#define IDC_BUI_INCREMENT_STATIC    53
#define IDC_BUI_INCREMENT           54
#define IDC_BUI_INC_LOG             55
#define IDC_BUI_MAXLOAD_STATIC      56
#define IDC_BUI_MAXLOAD             57
#define IDC_BUI_LOAD_INC_STATIC     58
#define IDC_BUI_LOAD_INC            59
#define IDC_BUI_FRAME_SMOOTHING_STATIC  60
#define IDC_BUI_FRAME_SMOOTHING         61
#define IDC_BUI_TAG                 62
#define IDC_BUI_MOTION              63

const LPWSTR g_meshes[] = 
{
    L"instancingtest_12.x",    
    L"instancingtest_24.x",    
    L"instancingtest_112.x",    
    L"instancingtest_264.x",    
    L"instancingtest_510.x",    
    L"instancingtest_1012.x",    
};

WCHAR g_FileName[MAX_PATH];

const int g_numMeshes = sizeof(g_meshes) / sizeof(LPWSTR);

int g_screenW = 0;
int g_screenH = 0;

bool bAutoSimulation = false;
bool bAutoExitAfterSimulation = false;

#define FRAME_SMOOTHING_FACTOR 3
void InitializeSimulationEvents();
void UpdateGUIFromSimulation();
void InitApp();
void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );

//--------------------------------------------------------------------------------------
// Reject any D3D10 devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
    return true;
}

//--------------------------------------------------------------------------------------
// Called right before creating a D3D9 or D3D10 device, allowing the app to modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext )
{
    pDeviceSettings->d3d10.SyncInterval = 0;
    return true;
}

//--------------------------------------------------------------------------------------
// Create any D3D10 resources that aren't dependant on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	HRESULT hr;
    V_RETURN( g_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( g_SettingsDlg.OnD3D10CreateDevice( pd3dDevice ) );
	V_RETURN( D3DX10CreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET, 
		OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, 
		L"Arial", &g_pFont10 ) );
	V_RETURN( D3DX10CreateSprite( pd3dDevice, 512, &g_pSprite10 ) );
	g_pTxtHelper = new CDXUTTextHelper( NULL, NULL, g_pFont10, g_pSprite10, 15 );

    g_instancing.Initialize(pd3dDevice);
    g_instancing.LoadNewMesh(g_SampleUI.GetListBox(IDC_MESHFILE)->GetSelectedItem()->strText);

	// Setup the camera's view parameters
	D3DXVECTOR3 vecEye(0.0f, 150.0f, -150.0f);
	D3DXVECTOR3 vecAt (0.0f, 0.0f, 0.0f);
	g_Camera.SetViewParams( &vecEye, &vecAt );
	g_Camera.SetEnablePositionMovement(true);
	g_Camera.SetScalers(0.01f,25.f);

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Create any D3D10 resources that depend on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	HRESULT hr;
	V_RETURN( g_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );
    V_RETURN( g_SettingsDlg.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );

    g_screenW = pBackBufferSurfaceDesc->Width;
    g_screenH = pBackBufferSurfaceDesc->Height;

	// Setup the camera's projection parameters
	float fAspectRatio = pBackBufferSurfaceDesc->Width / (FLOAT)pBackBufferSurfaceDesc->Height;
	g_Camera.SetProjParams( D3DX_PI/4, fAspectRatio, 0.1f, 1000.0f );

    // HUD is upper right
    g_HUD.SetSize( 170, 170 );
    g_HUD.SetLocation( pBackBufferSurfaceDesc->Width - g_HUD.GetWidth(), 0);

    // Main UI is upper left
    g_SampleUI.SetSize( 220, 400 );
    g_SampleUI.SetLocation( pBackBufferSurfaceDesc->Width-g_SampleUI.GetWidth(), pBackBufferSurfaceDesc->Height-g_SampleUI.GetHeight() );

    // Benchmark UI is used only when initializing a benchmark, placed bottom left
    g_BenchmarkUI.SetSize(220,400);
    g_BenchmarkUI.SetLocation( 20, pBackBufferSurfaceDesc->Height-g_BenchmarkUI.GetHeight() );
    

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Handle updates to the scene.  This is called regardless of which D3D API is used
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext )
{
	g_Camera.FrameMove( fElapsedTime );
    g_instancing.Update((float)fTime);

    if(bAutoSimulation)
    {
        g_testHarness.FrameUpdate(&g_instancing,fElapsedTime);

        if(g_testHarness.GetSimulationComplete())
            bAutoSimulation = false;
    }
}


//--------------------------------------------------------------------------------------
// Render the help and statistics text
//--------------------------------------------------------------------------------------
void RenderText()
{
	g_pTxtHelper->Begin();
	g_pTxtHelper->SetInsertionPos( 2, 0 );
	g_pTxtHelper->SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
	g_pTxtHelper->DrawTextLine( DXUTGetFrameStats(true) );
	g_pTxtHelper->DrawTextLine( DXUTGetDeviceStats() );

    WCHAR szSceneData[MAX_PATH];
    wsprintf(szSceneData,L"Draw Calls:%d , Polys Drawn:%d\n",g_numDrawCalls,g_instancing.NumInstances()*g_instancing.NumMeshPolys());
    g_pTxtHelper->DrawTextLine(szSceneData);

    if(bAutoSimulation)
    {
        wsprintf(szSceneData,L"** BENCHMARKING! ** UI Disabled.  DO NOT TASK SWITCH AWAY!! **");
        g_pTxtHelper->DrawTextLine(szSceneData);
    }
    else
    {
        wsprintf(szSceneData,L"* Free Roam Mode *");
        g_pTxtHelper->DrawTextLine(szSceneData);
    }

	g_pTxtHelper->End();

}

//--------------------------------------------------------------------------------------
// Render the scene using the D3D10 device
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
    if(bAutoSimulation)
    {
        //RecordSimulationEvent(fElapsedTime);
        g_testHarness.FrameRender(&g_instancing,fElapsedTime);
        UpdateGUIFromSimulation(); // process any changes to setting that might have happened based on test harness updates...
    }

    // Clear Render target and the depth stencil 
    float ClearColor[4] = { 0.0f, 0.0f, 0.0f, 0.0f };
    pd3dDevice->ClearRenderTargetView( DXUTGetD3D10RenderTargetView(), ClearColor );
    pd3dDevice->ClearDepthStencilView( DXUTGetD3D10DepthStencilView(), D3D10_CLEAR_DEPTH, 1.0, 0 );

	D3DXMATRIX  mView = *g_Camera.GetViewMatrix();
	D3DXMATRIX  mProj = *g_Camera.GetProjMatrix();

    g_numDrawCalls = g_instancing.Render(pd3dDevice,mView,mProj,fElapsedTime);

    g_HUD.OnRender( fElapsedTime );
    if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.OnRender( fElapsedTime );
        return;
    }

    RenderText();
	g_SampleUI.OnRender( fElapsedTime ); 
    g_BenchmarkUI.OnRender( fElapsedTime );
}

//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10ResizedSwapChain 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10ReleasingSwapChain( void* pUserContext )
{
	g_DialogResourceManager.OnD3D10ReleasingSwapChain();
    g_SettingsDlg.OnD3D10DestroyDevice();
}


//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10CreateDevice 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10DestroyDevice( void* pUserContext )
{
	g_DialogResourceManager.OnD3D10DestroyDevice();
    g_SampleUI.RemoveAllControls();
    g_BenchmarkUI.RemoveAllControls();
    g_HUD.RemoveAllControls();
	TextureLibrary::singleton()->Release();
    g_instancing.Release();

	SAFE_RELEASE( g_pFont10 );
	SAFE_RELEASE( g_pSprite10 );
	SAFE_DELETE( g_pTxtHelper );
}

//--------------------------------------------------------------------------------------
// Handle messages to the application
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, 
                          bool* pbNoFurtherProcessing, void* pUserContext )
{
    if(bAutoSimulation) return 0; // No user changing of params while in simulation

    if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
        return 0;
    }

	if(g_BenchmarkUI.MsgProc(hWnd,uMsg,wParam,lParam))
	{
        return 1;
	}

	if(g_SampleUI.MsgProc(hWnd,uMsg,wParam,lParam))
	{
        return 1;
	}

    if( g_HUD.MsgProc( hWnd, uMsg, wParam, lParam ) )
    {
        return 1;
    }

	if(g_Camera.HandleMessages( hWnd, uMsg, wParam, lParam ))
    {
        return 1;
    }

    if(uMsg == WM_KEYDOWN)
    {
        switch( (char)wParam )
        {
        case 'I':
        case 'i':
            {
                bool bInstancing = g_instancing.UseInstancing();
                bInstancing = !bInstancing;
                g_instancing.SetUseInstancing(bInstancing);
                g_SampleUI.GetCheckBox(IDC_USE_INSTANCING)->SetChecked(bInstancing);
                g_SampleUI.GetCheckBox(IDC_TRADITIONAL_INSTANCING)->SetEnabled(bInstancing);
            }
            break;

        case 'k':
        case 'l':
        case 'K':
        case 'L':

            if(wParam == 'k' || wParam == 'K')
			    g_instancing.SetNumInstanced(g_instancing.NumInstances() - 1);
            else
                g_instancing.SetNumInstanced(g_instancing.NumInstances() + 1);

            // Update the UI, etc
            g_SampleUI.GetSlider(IDC_NUM_INSTANCES)->SetValue(g_instancing.NumInstances()); 
            
            WCHAR sNumInstances[MAX_PATH];
            wsprintf(sNumInstances,L"Instances: %d",g_instancing.NumInstances());
            g_SampleUI.GetStatic(IDC_NUM_INSTANCES_STATIC)->SetText(sNumInstances);
            break;
        }

    }

    return 0;
}


//--------------------------------------------------------------------------------------
// Handle key presses
//--------------------------------------------------------------------------------------
void CALLBACK OnKeyboard( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext )
{
}


//--------------------------------------------------------------------------------------
// Handle mouse button presses
//--------------------------------------------------------------------------------------
void CALLBACK OnMouse( bool bLeftButtonDown, bool bRightButtonDown, bool bMiddleButtonDown, 
                       bool bSideButton1Down, bool bSideButton2Down, int nMouseWheelDelta, 
                       int xPos, int yPos, void* pUserContext )
{
}


//--------------------------------------------------------------------------------------
// Call if device was removed.  Return true to find a new device, false to quit
//--------------------------------------------------------------------------------------
bool CALLBACK OnDeviceRemoved( void* pUserContext )
{
    return true;
}


//--------------------------------------------------------------------------------------
// Initialize everything and go into a Render loop
//--------------------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // DXUT will create and use the best device (either D3D9 or D3D10) 
    // that is available on the system depending on which D3D callbacks are set below

    // Set general DXUT callbacks
    DXUTSetCallbackFrameMove( OnFrameMove );
    DXUTSetCallbackKeyboard( OnKeyboard );
    DXUTSetCallbackMouse( OnMouse );
    DXUTSetCallbackMsgProc( MsgProc );
    DXUTSetCallbackDeviceChanging( ModifyDeviceSettings );
    DXUTSetCallbackDeviceRemoved( OnDeviceRemoved );

   // Set the D3D10 DXUT callbacks. Remove these sets if the app doesn't need to support D3D10
    DXUTSetCallbackD3D10DeviceAcceptable( IsD3D10DeviceAcceptable );
    DXUTSetCallbackD3D10DeviceCreated( OnD3D10CreateDevice );
    DXUTSetCallbackD3D10SwapChainResized( OnD3D10ResizedSwapChain );
    DXUTSetCallbackD3D10FrameRender( OnD3D10FrameRender );
    DXUTSetCallbackD3D10SwapChainReleasing( OnD3D10ReleasingSwapChain );
    DXUTSetCallbackD3D10DeviceDestroyed( OnD3D10DestroyDevice );

	HRESULT hr;
	V_RETURN(DXUTSetMediaSearchPath(L"..\\Source\\InstancingTests"));

    // Perform any application-level initialization here
    std::wstring sCommandLine = lpCmdLine;

    if(sCommandLine.find(L"-benchmark") != std::string::npos)
    {
        bAutoSimulation = true;
        bAutoExitAfterSimulation = true;
    }
    InitApp();
    DXUTInit( true, true, NULL ); // Parse the command line, show msgboxes on error, no extra command line params
    DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
    DXUTCreateWindow( L"InstancingTests" );
    DXUTCreateDevice( true, 1024, 768 );  
    DXUTMainLoop(); // Enter into the DXUT Render loop

    // Perform any application-level cleanup here

    return DXUTGetExitCode();
}

void InitApp()
{
    int iX = 0;
    int iY = 0;

    g_SampleUI.Init( &g_DialogResourceManager );
    g_SampleUI.SetCallback( OnGUIEvent );
    WCHAR sNumInstances[MAX_PATH];
    wsprintf(sNumInstances,L"Instances: %d",g_instancing.NumInstances());
    g_SampleUI.AddStatic(IDC_NUM_INSTANCES_STATIC,sNumInstances,iX,iY,170,15);
    g_SampleUI.AddSlider(IDC_NUM_INSTANCES,iX,iY+=15,170,15,1,g_instancing.NumMaxInstances(),g_instancing.NumInstances());

    WCHAR sCPULoad[MAX_PATH];
    wsprintf(sCPULoad,L"CPULoad(ms): %d ms",g_instancing.GetCPULoad());
    g_SampleUI.AddStatic(IDC_CPU_LOAD_STATIC,sCPULoad,iX,iY+=15,170,15);
    g_SampleUI.AddSlider(IDC_CPU_LOAD,iX,iY+=15,170,15,0,100,g_instancing.GetCPULoad());

    g_SampleUI.AddCheckBox(IDC_USE_INSTANCING,L"Instancing?",iX,iY+=15,170,15,g_instancing.UseInstancing());
    g_SampleUI.AddCheckBox(IDC_TRADITIONAL_INSTANCING,L"Stream Instancing?",iX+5,iY+=15,170,12,g_instancing.TraditionalInstancing());

    g_SampleUI.AddCheckBox(IDC_MOTION_ACTIVE,L"Motion?(adds CPU Load and Bandwidth)",iX,iY+=15,190,15,g_instancing.MotionActive());

    iY+=5;  // a little gap

    g_SampleUI.AddStatic(IDC_SCALE_STATIC,L"Model Scale",iX,iY+=15,170,15);
    g_SampleUI.AddSlider(IDC_SCALE,iX,iY+=15,170,15,1,1000,100);

    iY+=5;  // a little gap
    g_SampleUI.AddButton(IDC_OPENMESHFILE,L"Load Mesh File...",iX,iY+=15,150,15);

    iY+=5;  // a little gap
    g_SampleUI.AddStatic(IDC_MESHFILE_STATIC,L"*Preset Meshes*",iX,iY+=15,170,15);

    CDXUTListBox *pListBox = NULL;
    g_SampleUI.AddListBox(IDC_MESHFILE,iX,iY+=15,170,100,0,&pListBox);
    for(int i=0;i<g_numMeshes;i++)
    {
        pListBox->AddItem(g_meshes[i],NULL);
    }
    pListBox->SelectItem(0);

    g_SampleUI.AddButton(IDC_BENCHMARKSTART,L"Benchmark Mesh...",iX,iY+=110,150,15);

    // Benchmark UI
    g_BenchmarkUI.Init( &g_DialogResourceManager );
    g_BenchmarkUI.SetCallback( OnGUIEvent ); iY = 10;

    g_BenchmarkUI.AddCheckBox(IDC_BUI_MOTION,L"Enable Motion?",iX,iY,170,15);

    wsprintf(sNumInstances,L"Max Instances: %d",g_instancing.NumMaxInstances());
    g_BenchmarkUI.AddStatic(IDC_BUI_MAXINST_STATIC,sNumInstances,iX,iY+=15,170,15);
    g_BenchmarkUI.AddSlider(IDC_BUI_MAXINST,iX,iY+=15,170,15,1,g_instancing.NumMaxInstances(),g_instancing.NumMaxInstances());

    g_BenchmarkUI.AddCheckBox(IDC_BUI_INC_LOG,L"Logarithmic Increments?",iX,iY+=15,170,15,true);

    CDXUTStatic*pStatic = NULL;
    g_BenchmarkUI.AddStatic(IDC_BUI_INCREMENT_STATIC,L"Linear Steps: 100",iX,iY+=15,170,15,false,&pStatic);
    pStatic->SetEnabled(false);
    CDXUTSlider *pIncrement = NULL;
    g_BenchmarkUI.AddSlider(IDC_BUI_INCREMENT,iX,iY+=15,170,15,1,g_instancing.NumMaxInstances(),100,false,&pIncrement);
    pIncrement->SetEnabled(false);

    wsprintf(sCPULoad,L"Max Load(ms): %d ms",0);
    g_BenchmarkUI.AddStatic(IDC_BUI_MAXLOAD_STATIC,sCPULoad,iX,iY+=15,170,15);
    g_BenchmarkUI.AddSlider(IDC_BUI_MAXLOAD,iX,iY+=15,170,15,0,100,0);

    wsprintf(sCPULoad,L"Load Steps: %d",10);
    g_BenchmarkUI.AddStatic(IDC_BUI_LOAD_INC_STATIC,sCPULoad,iX,iY+=15,170,15);
    g_BenchmarkUI.AddSlider(IDC_BUI_LOAD_INC,iX,iY+=15,170,15,0,100,10);

    wsprintf(sCPULoad,L"Smoothing Loop: %d",FRAME_SMOOTHING_FACTOR);
    g_BenchmarkUI.AddStatic(IDC_BUI_FRAME_SMOOTHING_STATIC,sCPULoad,iX,iY+=15,170,15);
    g_BenchmarkUI.AddSlider(IDC_BUI_FRAME_SMOOTHING,iX,iY+=15,170,15,0,100,FRAME_SMOOTHING_FACTOR);

    g_BenchmarkUI.AddEditBox(IDC_BUI_TAG,L"BenchmarkName",iX,iY+=15,170,30);

    g_BenchmarkUI.AddButton(IDC_BUI_START,L"Run Benchmark",iX,iY+=30,170,15);

    g_BenchmarkUI.SetVisible(false);

    g_SettingsDlg.Init( &g_DialogResourceManager );
    g_HUD.Init( &g_DialogResourceManager );

    g_HUD.SetCallback( OnGUIEvent ); iY = 10;
    g_HUD.AddButton( IDC_TOGGLEFULLSCREEN, L"Toggle full screen", 35, iY, 125, 22 );
    g_HUD.AddButton( IDC_TOGGLEREF, L"Toggle REF (F3)", 35, iY += 24, 125, 22, VK_F3 );
    g_HUD.AddButton( IDC_CHANGEDEVICE, L"Change device (F2)", 35, iY += 24, 125, 22, VK_F2 );

    if(bAutoSimulation)
    {
        InitializeSimulationEvents();
    }
}

void InitializeSimulationEvents()
{
    g_SampleUI.GetCheckBox(IDC_MOTION_ACTIVE)->SetChecked(g_BenchmarkUI.GetCheckBox(IDC_BUI_MOTION)->GetChecked());
    g_instancing.SetMotionActive(g_BenchmarkUI.GetCheckBox(IDC_BUI_MOTION)->GetChecked());

    DWORD maxInstances = (DWORD)g_BenchmarkUI.GetSlider(IDC_BUI_MAXINST)->GetValue();
    DWORD maxLoad = (DWORD)g_BenchmarkUI.GetSlider(IDC_BUI_MAXLOAD)->GetValue();
    DWORD instSteps = (DWORD)g_BenchmarkUI.GetSlider(IDC_BUI_INCREMENT)->GetValue();
    DWORD loadSteps = (DWORD)g_BenchmarkUI.GetSlider(IDC_BUI_LOAD_INC)->GetValue();

    bool bLogIncrement = g_BenchmarkUI.GetCheckBox(IDC_BUI_INC_LOG)->GetChecked();
    DWORD frameSmooth = (DWORD)g_BenchmarkUI.GetSlider(IDC_BUI_FRAME_SMOOTHING)->GetValue();
    LPCWSTR wzTag = g_BenchmarkUI.GetEditBox(IDC_BUI_TAG)->GetText();

    g_testHarness.InitializeExperiments(&g_instancing,maxInstances,maxLoad,instSteps,loadSteps,bLogIncrement,frameSmooth,wzTag);
}

void UpdateGUIFromSimulation()
{
    WCHAR sNumInstances[MAX_PATH];
    wsprintf(sNumInstances,L"Instances: %d",g_instancing.NumInstances());
    g_SampleUI.GetStatic(IDC_NUM_INSTANCES_STATIC)->SetText(sNumInstances);
    g_SampleUI.GetSlider(IDC_NUM_INSTANCES)->SetValue(g_instancing.NumInstances());

    WCHAR sCPULoad[MAX_PATH];
    wsprintf(sCPULoad,L"CPULoad(ms): %d ms",g_instancing.GetCPULoad());
    g_SampleUI.GetStatic(IDC_CPU_LOAD_STATIC)->SetText(sCPULoad);
    g_SampleUI.GetSlider(IDC_CPU_LOAD)->SetValue(g_instancing.GetCPULoad());

    g_SampleUI.GetCheckBox(IDC_USE_INSTANCING)->SetChecked(g_instancing.UseInstancing());
    g_SampleUI.GetCheckBox(IDC_TRADITIONAL_INSTANCING)->SetChecked(g_instancing.TraditionalInstancing());

}

//--------------------------------------------------------------------------------------
// Handles the GUI events
//--------------------------------------------------------------------------------------
void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext )
{
    if(bAutoSimulation) return; // No user interaction with GUI while in simulation

	switch( nControlID )
	{
    case IDC_TOGGLEFULLSCREEN: DXUTToggleFullScreen(); break;
    case IDC_TOGGLEREF:        DXUTToggleREF(); break;
    case IDC_CHANGEDEVICE:     g_SettingsDlg.SetActive( !g_SettingsDlg.IsActive() ); break;

	case IDC_NUM_INSTANCES:    
		{
            g_instancing.SetNumInstanced(g_SampleUI.GetSlider(IDC_NUM_INSTANCES)->GetValue());
            
            WCHAR sNumInstances[MAX_PATH];
            wsprintf(sNumInstances,L"Instances: %d",g_instancing.NumInstances());
            g_SampleUI.GetStatic(IDC_NUM_INSTANCES_STATIC)->SetText(sNumInstances);
		}
		break;
	case IDC_USE_INSTANCING:    
		{
			g_instancing.SetUseInstancing(g_SampleUI.GetCheckBox(IDC_USE_INSTANCING)->GetChecked());
            g_SampleUI.GetCheckBox(IDC_TRADITIONAL_INSTANCING)->SetEnabled(g_instancing.UseInstancing());
		}
		break;
    case IDC_TRADITIONAL_INSTANCING:
		{
			g_instancing.SetTraditionalInstancing(g_SampleUI.GetCheckBox(IDC_TRADITIONAL_INSTANCING)->GetChecked());
		}
		break;

	case IDC_MOTION_ACTIVE:    
		{
			g_instancing.SetMotionActive(g_SampleUI.GetCheckBox(IDC_MOTION_ACTIVE)->GetChecked());
		}
		break;
	case IDC_CPU_LOAD:    
		{
            g_instancing.SetCPULoad(g_SampleUI.GetSlider(IDC_CPU_LOAD)->GetValue());

            WCHAR sCPULoad[MAX_PATH];
            wsprintf(sCPULoad,L"CPULoad(ms): %d ms",g_instancing.GetCPULoad());
            g_SampleUI.GetStatic(IDC_CPU_LOAD_STATIC)->SetText(sCPULoad);
		}
		break;

    case IDC_MESHFILE:
        {
            if(g_activeMesh != g_SampleUI.GetListBox(IDC_MESHFILE)->GetSelectedIndex())
            {
                g_activeMesh = g_SampleUI.GetListBox(IDC_MESHFILE)->GetSelectedIndex();
                StringCchCopy(g_FileName,MAX_PATH,g_SampleUI.GetListBox(IDC_MESHFILE)->GetSelectedItem()->strText);
                g_instancing.LoadNewMesh(g_FileName);
            }
        }
        break;

    case IDC_OPENMESHFILE:
        {
            OPENFILENAME info;
            ZeroMemory(&info, sizeof (OPENFILENAME));
            info.lStructSize = sizeof (OPENFILENAME);
            info.hwndOwner = DXUTGetHWND();
            info.lpstrFilter = L"X file meshes (.x)\0*.x\0\0";
            info.nFilterIndex = 1;
            info.lpstrFile = (LPWSTR)g_FileName;
            info.nMaxFile = 256;
            info.lpstrTitle = L"Open";
            info.nMaxFileTitle = sizeof(info.lpstrTitle);
            info.dwReserved = OFN_EXPLORER;

            if (GetOpenFileName(&info))
            {
                // mesh create code
                g_instancing.LoadNewMesh((LPWSTR)g_FileName);
            }
        }
        break;

    case IDC_SCALE:
        {
            g_instancing.SetScale((float)g_SampleUI.GetSlider(IDC_SCALE)->GetValue() / 100.f);
        }
        break;

    case IDC_BENCHMARKSTART:
        {
            g_BenchmarkUI.SetVisible(!g_BenchmarkUI.GetVisible());
        }
        break;



/*
IDC_BUI_START               50
*/
    case IDC_BUI_MAXINST:    
		{
            int num = ((CDXUTSlider*)pControl)->GetValue();
            
            WCHAR sNum[MAX_PATH];
            wsprintf(sNum,L"Max Instances: %d",num);
            g_BenchmarkUI.GetStatic(IDC_BUI_MAXINST_STATIC)->SetText(sNum);

            g_BenchmarkUI.GetSlider(IDC_BUI_INCREMENT)->SetRange(1,num);

            num = g_BenchmarkUI.GetSlider(IDC_BUI_INCREMENT)->GetValue();
            
            wsprintf(sNum,L"Linear Steps: %d",num);
            g_BenchmarkUI.GetStatic(IDC_BUI_INCREMENT_STATIC)->SetText(sNum);
		}
		break;

    case IDC_BUI_INCREMENT:    
		{
            int num = ((CDXUTSlider*)pControl)->GetValue();
            
            WCHAR sNum[MAX_PATH];
            wsprintf(sNum,L"Linear Steps: %d",num);
            g_BenchmarkUI.GetStatic(IDC_BUI_INCREMENT_STATIC)->SetText(sNum);
		}
		break;

    case IDC_BUI_MAXLOAD:    
		{
            int num = ((CDXUTSlider*)pControl)->GetValue();
            
            WCHAR sNum[MAX_PATH];
            wsprintf(sNum,L"Max Load(ms): %d ms",num);
            g_BenchmarkUI.GetStatic(IDC_BUI_MAXLOAD_STATIC)->SetText(sNum);

            g_BenchmarkUI.GetSlider(IDC_BUI_LOAD_INC)->SetRange(1,num);

            num = g_BenchmarkUI.GetSlider(IDC_BUI_LOAD_INC)->GetValue();
            
            wsprintf(sNum,L"Load Steps: %d",num);
            g_BenchmarkUI.GetStatic(IDC_BUI_LOAD_INC_STATIC)->SetText(sNum);

		}
		break;

    case IDC_BUI_LOAD_INC:    
		{
            int num = ((CDXUTSlider*)pControl)->GetValue();
            
            WCHAR sNum[MAX_PATH];
            wsprintf(sNum,L"Load Steps: %d",num);
            g_BenchmarkUI.GetStatic(IDC_BUI_LOAD_INC_STATIC)->SetText(sNum);
		}
		break;

    case IDC_BUI_INC_LOG:
        {
            bool bLogInc = ((CDXUTCheckBox*)pControl)->GetChecked();

            g_BenchmarkUI.GetStatic(IDC_BUI_INCREMENT_STATIC)->SetEnabled(!bLogInc);
            g_BenchmarkUI.GetSlider(IDC_BUI_INCREMENT)->SetEnabled(!bLogInc);
        }
        break;

    case IDC_BUI_FRAME_SMOOTHING:    
		{
            int num = ((CDXUTSlider*)pControl)->GetValue();
            
            WCHAR sNum[MAX_PATH];
            wsprintf(sNum,L"Smoothing Loop: %d",num);
            g_BenchmarkUI.GetStatic(IDC_BUI_FRAME_SMOOTHING_STATIC)->SetText(sNum);
		}
		break;

    case IDC_BUI_START:
        {
            g_BenchmarkUI.SetVisible(false);
            bAutoSimulation = true;
            InitializeSimulationEvents();
        }
        break;
	}
}


